/***************************************************************************
                          komportview.h  -  Komport Serial Port Communicator
                             -------------------
    begin                : Mon Feb 17 00:05:54 EST 2003
    copyright            : (C) 2003 by Mike Sharkey
    email                : michael@sharkey.servebeer.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KOMPORTVIEW_H
#define KOMPORTVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for Qt
#include <qapplication.h>
#include <qclipboard.h>
#include <qwidget.h>
#include <qscrollview.h>
#include <qrect.h>
#include <qpixmap.h>
#include <qscrollbar.h>

#include "komportserial.h"
#include "komportemulation.h"

#include <stdlib.h>

class KomportDoc;
class KomportEmulation;

/** The KomportView class provides the view widget for the KomportApp instance.	
 * The View instance inherits QWidget as a base class and represents the view object of a KTMainWindow. As KomportView is part of the
 * docuement-view model, it needs a reference to the document object connected with it by the KomportApp class to manipulate and display
 * the document structure provided by the KomportDoc class.
 * 	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 0.4 code generation
 */
class KomportView : public QScrollView
{
  Q_OBJECT
  public:
    /** Constructor for the main view */
    KomportView(QWidget *parent = 0, const char *name=0);
    /** Destructor for the main view */
    ~KomportView();

    /** returns a pointer to the document connected to the view instance. Mind that this method requires a KomportApp instance as a parent
     * widget to get to the window document pointer by calling the KomportApp::getDocument() method.
     *
     * @see KomportApp#getDocument
     */
    KomportDoc *getDocument() const;

    /** contains the implementation for printing functionality */
    void print(QPrinter *pPrinter);
  /** initialize settings */
  void initSettings();
  /** Re-draw a cell given and _x, _y coordinate. */
  void updateCell(int _x,int _y);
  /** Re-draw a cell given a QPoint coordinate. */
  void updateCell(QPoint _p);
  /** Re-draw a cell given and _x, _y coordinate. */
  void updateCell(QPainter* _p, int _x,int _y);
  /** Re-draw a cell given a QPoint coordinate. */
  void updateCell(QPainter* _p, QPoint _p);
  /** Get a pointer to the serial port object. */
  KomportSerial* getSerial();
  /** clear selection */
  void deselect();
  /** clear selection outside of bounds */
  void deselect(QRect _bounds);
  /** make a selection by cell coordinates*/
  void select(QPoint start, QPoint end, bool clip=false);
  /** has a text selection */
  bool hasSelection();
  /** set scroll buffer size */
  void setScrollBuffer( int _sz);
  /** retrieve blink property */
  inline bool blink()                 {return mBlink;}
  /** retrieve bold property */
  inline bool bold()                  {return mBold;}
  /** retrieve reverse property */
  inline bool reverse()               {return mReverse;}
  /** retrieve underline property */
  inline bool underline()             {return mUnderline;}
  /** get background color */
  inline QColor backgroundColor()     {return mBackgroundColor;}
  /** get foreground color */
  inline QColor foregroundColor()     {return mForegroundColor;}

  /** set blink property */
  inline void setBlink(bool _b)       {mBlink=_b;}
  /** set bold property */
  inline void setBold(bool _b)        {mBold=_b;}
  /** set reverse property */
  inline void setReverse(bool _b)     {mReverse = _b;}
  /** set underline property */
  inline void setUnderline(bool _b)   {mUnderline = _b;}
  /** set forground color */
  inline void setForegroundColor(QColor _c) {mForegroundColor = _c;}
  /** set background color */
  inline void setBackgroundColor(QColor _c) {mBackgroundColor = _c;}

  /** set the cell array size .
 */
  virtual void setArraySize(QSize _sz);
  /** set cell size */
  void setCellSize(QSize _sz);
  /** return calculated dimensions in pixels based on cell size and array size */
  QSize size();
  /** get cell size */
  QSize cellSize();
  /** get array size */
  QSize arraySize();
  /** return calculated height based on cell height * array height */
  int height();
  /** return calculated width based on cell width * array width */
  int width();
  /** cell width */
  int cellWidth();
  /** array height */
  int arrayHeight();
  /** array width  */
  int arrayWidth();
  /** cell heiht */
  int cellHeight();
  /** draw a character into a cell */
  void drawChar(QChar _c,int _x, int _y);
  /** draw a character into a cell */
  void drawChar(QChar _c,QPoint _p);
  /** get a pointer to the cell from location (x,y) */
  virtual KomportCell* getCell(int _x,int _y);
  /** get a pointer to the cell from location (x,y) */
  virtual KomportCell* getCell(QPoint _p);
  /** cursor position */
  void setCursor(int _x, int _y) {setCursor(QPoint(_x,_y));}
  /** cursor position */
  virtual void setCursor(QPoint _p);
  /** cursor position */
  virtual QPoint cursor();
  /** Insert a character at the current cursor position and advance cursor. Scroll if advance is passed the last column. */
  void putChar(QChar _ch);
  /** move the cursor once cell forward, scolling or wrapping as nessesary */
  void advanceCursor();
  /** retrieve a character from a cell */
  QChar getChar(QPoint _p);
  /** scroll up one line */
  virtual void scrollUp();
  /** clear a row */
  void clearRow(int _row);
  /** update all cells */
  void updateAll();
  /** update a row */
  void updateRow(int _row);
  /** clear all cells */
  void clear();
  /** default background color */
  QColor defaultBackgroundColor();
  /** default foreground color */
  QColor defaultForegroundColor();
  /** set cell attributes */
  void setCellAttributes(QPoint _p);
  /** clear to end of line */
  void clearEOL();
  /** clear cell */
  void clear(QPoint _p);
  /** clear cell */
  void clear(int _x, int _y);
  /** set a cell */
  virtual void setCell( QPoint _p, KomportCell* _cell);
  /** copy a row  */
  void copyRow(int _dst, int _src);
  /** return a rectangle in cell coordinates of the visible cells */
  QRect contentCellsRect();
protected:
  /** terminal emulation */
  KomportEmulation* mEmulation;
  /** notify cursor position has changde */
  void cursorChanged(QPoint _old, QPoint _new);
  /** This event handler is called whenever the QScrollView receives a mousePressEvent(): the press position in e is translated to be a point on the contents. */
  virtual void contentsMousePressEvent( QMouseEvent* _e );
  /**  This event handler is called whenever the QScrollView receives a mousePressEvent(): the press position in e is translated to be a point on the contents. */
  virtual void contentsMouseReleaseEvent( QMouseEvent* _e );
  /** This event handler is called whenever the QScrollView receives a mouseMoveEvent(): the mouse position in e is translated to be a point on the contents. */
  virtual void contentsMouseMoveEvent( QMouseEvent* _e );
  /** key press event */
  virtual void keyPressEvent(QKeyEvent* _e);
  /** key release event */
  virtual void keyReleaseEvent(QKeyEvent* _e);
  /** timer event */
  virtual void timerEvent(QTimerEvent* _e);
  /** draw a cell */
  virtual void paintCell( QPainter* _paint, KomportCell* _cell, QRect _bounds, bool _cursorOn=false); 
  /** The function should draw the rectangle (clipx, clipy, clipw, cliph) of the contents using painter p. The clip rectangle is in the scrollview's coordinates.
  The clip rectangle and translation of the painter p is already set appropriately. */
  virtual void  drawContents  ( QPainter* _paint, int clipx, int clipy, int clipw, int cliph );
  /** calculates cell size based on font. */
  void setCellSize();
  /** begin a selection by pixel coordinate */
  void selectStart( QPoint _pt );
  /** end a selection by pixel coordinate */
  void selectEnd( QPoint _pt );
  /** reset scrollbar */
  void resetScroll();
  /** convert a local pixel coord to a cell coord */
  QPoint cellPoint(QPoint _pt);
  /** Returns true if the specified cell is the cursor cell, and the current cursor state is true, and cursor is enabled.
 */
  bool cursorCellOn(int _x, int _y);
public slots: // Public slots
  /** received a char */
  void slotReceivedChar(char _ch);
  /** No descriptions */
  void slotSelectionChanged();
  /** simulated key press for inserting from clipboard, etc... */
  void slotSimKeyPressed(QChar _c);
  /** rx queue has overflowed */
  virtual void slotQueueOverflow();
  /** turn cursor off (invisible) */
  virtual void slotCursorOff();
  /** turn cursor on (visible) */
  virtual void slotCursorOn();
signals: // Signals
  /** No descriptions */
  virtual void viewModified(KomportView*);
  /** key pressed */
  virtual void keyPressed(QKeyEvent*);
  /** simulated key press */
  virtual void simKeyPressed(QChar);
  /** update the status string */
  virtual void updateStatus(const QString &);
private: // Private attributes
  /** size of cell array */
  QSize mArraySize;
  /** cell list */
  QPtrList<KomportCell> mCells;
  /** size of cell in pixels*/
  QSize mCellSize;
  /** cusror position */
  QPoint mCursor;
  /** blink property */
  bool mBlink;
  /** bold property */
  bool mBold;
  /** reverse video property */
  bool mReverse;
  /** underline property */
  bool mUnderline;
  /** forground color */
  QColor mForegroundColor;
  /** background color */
  QColor mBackgroundColor;
  /** if true, signals are emitted when a cell is changed, otherwise not */
  bool mNotify;
  /** cursor is visible? */
  bool mCursorOn;
  /** cursor state */
  bool mCursorState;
  /** blink state */
  bool mBlinkState;
  /** cursor time */
  int mCursorTimer;
  /** blink timer */
  int mBlinkTimer;
  /** performing a text selection */
  bool mInSelection;
  /** has text selection */
  bool mHasSelection;
  /** auto scroll timer for extending text selection into scroll buffer */
  int mAutoScrollTimer;
  /** for tracking mouse during selctions */
  QPoint mMousePos;
  /** updates the status string */
  int mStatusTimer;
  /** text selection rectangle */
  QRect mSelectRect;
  /** Selection anchor */
  QPoint mAnchor;
  /** currently executing a timer-scroll operation */
  bool mInTimerScroll;
};

#endif // KOMPORTVIEW_H
